use ExtUtils::MakeMaker;
use File::Basename;
use blib '..';
use blib '../GdkImlib';
use blib '../Gnome';
use Gtk::Install::Files;
use Gtk::Gdk::ImlibImage::Install::Files;
use Gnome::Install::Files;
use CORBA::ORBit::Install::Files;
use Cwd;
require '../Depends.pm';
import ExtUtils::Depends;

require '../tools/gendefs.pl';

mkdir('build', 0777);

$pm = {};

$inc = `gnome-config --cflags bonobo`;
chomp($inc);
$libs = `gnome-config --libs bonobo`;
chomp($libs);

$typemaps = [cwd() . '/pkg.typemap', cwd().'/build/GnomeBonoboTypemap'];
@headers = ('build/GnomeBonoboDefs.h');


ExtUtils::Depends::save_config('Gnome::Bonobo', 'build/IFiles.pm', $inc, $libs, 
	$typemaps, [keys %defs], [@defines], [@headers], $pm);

foreach (@Gtk::Install::Files::typemaps) {
	push(@$typemaps, cwd().'/'.$Gtk::Install::Files::CORE . '/'. $_);
}
foreach (@Gtk::Gdk::ImlibImage::Install::Files::typemaps) {
	push(@$typemaps, cwd().'/'.$Gtk::Gdk::ImlibImage::Install::Files::CORE . '/'. $_);
}
foreach (@Gnome::Install::Files::typemaps) {
	push(@$typemaps, cwd().'/'.$Gnome::Install::Files::CORE . '/'. $_);
}
foreach (@CORBA::ORBit::Install::Files::typemaps) {
	push(@$typemaps, $CORBA::ORBit::Install::Files::CORE . '/'. $_);
}
push(@defines, @Gnome::Install::Files::defines, @Gtk::Install::Files::defines, @Gtk::Gdk::ImlibImage::Install::Files::defines);
#push(@defines, '-I../build', '-Ibuild', '-I.', '-I../Gtk');
push(@defines, "-I$CORBA::ORBit::Install::Files::CORE");
push(@defines, "-I$Gnome::Install::Files::CORE");
push(@defines, "-I$Gtk::Gdk::ImlibImage::Install::Files::CORE");
push(@defines, '-Ibuild', '-I.', "-I$Gtk::Install::Files::CORE");

$pm->{'Bonobo.pm'} = '$(INST_ARCHLIBDIR)/Gnome/Bonobo.pm';
$pm->{'build/GnomeBonoboTypes.pm'} = '$(INST_ARCHLIBDIR)/Gnome/Bonobo/Types.pm';

add_c ('build/GnomeBonoboDefs.c');
add_defs ('pkg.defs');
add_headers (qw( <bonobo/gnome-bonobo.h> ));
add_raw_xs('xs/Bonobo.xs');

#add_raw_xs ('xs/Gnome.xs', 'xs/GnomeDialogUtil.xs', 'xs/GnomeDNS.xs', 
#	'xs/GnomeGeometry.xs', 'xs/GnomeICE.xs');

@a = (	'-f', 'GnomeBonobo', 
		'-p', 'Gtk=Gtk',
		'-p', 'GdkGL=Gtk::Gdk::GL',
		'-p', 'GdkRgb=Gtk::Gdk::Rgb',
		'-p', 'Gdk=Gtk::Gdk',
		'-p', 'Gnome=Gnome',
		'-p', 'Panel=Gnome::Panel',
		'-m', 'Gnome::Bonobo');

add_headers (@CORBA::ORBit::Install::Files::headers);
add_headers (@Gnome::Install::Files::headers);
add_headers (@Gtk::Install::Files::headers);
add_headers (@Gtk::Gdk::ImlibImage::Install::Files::headers);
# FIXME: use defs and headers from ExtUtils::Depends....
%defs = %ExtUtils::Depends::defs;
%headers = %ExtUtils::Depends::headers;
foreach (sort {$defs{$a} <=> $defs{$b}} keys %defs) {
	push @a, '-d', $_;
}
foreach (sort {$headers{$a} <=> $headers{$b}} keys %headers) {
	push @a, '-i', $_;
}

add_raw_xs(gendefs::gendefs(@a));

($xfiles, $object, $ldfrom) = setup_xs();

$depend = { 'build/GnomeBonoboDefs.c' => 'build/GnomeBonoboDefs.h build/PerlGnomeBonoboInt.h' };

foreach (qw(build/GnomeBonoboDefs.h build/PerlGnomeBonoboInt.h)) {
	$pm->{$_} = '$(INST_ARCHLIBDIR)/Gnome/Bonobo/Install/'. basename($_);
}

write_ext();

@clean = qw(
	build/IFiles.pm build/GnomeBonoboDefs.c build/GnomeBonoboDefs.h 
	build/GnomeBonoboTypemap build/GnomeBonoboTypes.pm 
	build/PerlGnomeBonoboExt.c build/PerlGnomeBonoboExt.h 
	build/PerlGnomeBonoboInt.h build/boxed.xsh build/extension.xsh 
	build/objects.xsh build/perl-bonobo-ds.pod build/perl-bonobo-ref.pod
);
push(@clean, @ExtUtils::Depends::clean);

# documentation
gendefs::gen_doc('bonobo');
system("../tools/gendoc.pl", "-t", 'bonobo', keys %ExtUtils::Depends::xs);


WriteMakefile(
	'NAME'      => 'Gnome::Bonobo',
	'VERSION_FROM'	=> 'Bonobo.pm',
	'PM' => $pm,
	'TYPEMAPS' => $typemaps,
	'XS' => $xfiles,
	'XSOPT' => '-noversioncheck',
	'DEFINE'    => join(' ',@defines),
	'dist' => { COMPRESS=>"gzip", SUFFIX=>"gz" },
	'INC' => $inc,
	'LIBS' => [$libs],
	'OBJECT' => $object,
	'OPTIMIZE' => '-O2 -g',
	'clean' => {FILES => join(' ', @clean) },
	#'LDFROM' => $ldfrom,
	'depend' => $depend,
);

sub MY::postamble {

'
Makefile: ' . join(' ',
	sort {$defs{$a} <=> $defs{$b}} keys %defs,
	<*/pkg*.pl>
	) . '

build/PerlGnomeBonoboExt.h build/PerlGnomeBonoboExt.c build/PerlGnomeBonoboInt.h: build/GnomeBonoboDefs.h ../tools/genext.pl
	$(PERL) ../tools/genext.pl GnomeBonobo </dev/null

';



}

sub MY::c_o {
	package MY; # so that "SUPER" works right
	my $inherited = shift->SUPER::c_o(@_);
	$inherited =~ s/CCCMD.*$/$&\n\t\@if test -f `basename \$*.o` -a "`basename \$*.o`" != "\$*.o"; then mv `basename \$*.o` \$*.o; fi/m;
    $inherited;
}

sub MY::const_config
{
	package MY;
	my $self = shift;
	my $flags = $self->{'CCCDLFLAGS'};
	$flags =~ s/(-[fK]?\s*)pic\b/${1}PIC/;
	$self->{'CCCDLFLAGS'} = $flags;
	return $self->SUPER::const_config;
}

	
