
#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#include "PerlGtkExt.h"
#include "PerlGnomeBonoboInt.h"

#include "GnomeBonoboDefs.h"

static GnomeView *
generic_view_factory (GnomeEmbeddable *embeddable, const GNOME_ViewFrame view_frame, void *closure) {
	AV * args = (AV*)closure;
	SV *handler = * av_fetch(args, 0, 0);
	int i;
	SV * result;
	dSP;

	ENTER;
	SAVETMPS;
	PUSHMARK(sp);
	XPUSHs(sv_2mortal(newSVGtkObjectRef(embeddable, 0)));
	XPUSHs(sv_2mortal(newSVsv(porbit_objref_to_sv(view_frame))));
	for (i=1;i<=av_len(args);i++)
		XPUSHs(sv_2mortal(newSVsv(*av_fetch(args, i, 0))));
	PUTBACK;
	i = perl_call_sv(handler, G_SCALAR);
	SPAGAIN;
	if (i!=1)
		croak("handler failed");
	result = POPs;
	PUTBACK;
	FREETMPS;
	LEAVE;

	return GNOME_VIEW(SvGtkObjectRef(result, 0));
}

static GnomeCanvasComponent *
generic_canvas_factory (GnomeEmbeddable *embeddable, GnomeCanvas *canvas, void *closure) {
	AV * args = (AV*)closure;
	SV *handler = * av_fetch(args, 0, 0);
	int i;
	SV * result;
	dSP;

	ENTER;
	SAVETMPS;
	PUSHMARK(sp);
	XPUSHs(sv_2mortal(newSVGtkObjectRef(embeddable, 0)));
	XPUSHs(sv_2mortal(newSVGtkObjectRef(canvas, 0)));
	for (i=1;i<=av_len(args);i++)
		XPUSHs(sv_2mortal(newSVsv(*av_fetch(args, i, 0))));
	PUTBACK;
	i = perl_call_sv(handler, G_SCALAR);
	SPAGAIN;
	if (i!=1)
		croak("handler failed");
	result = POPs;
	PUTBACK;
	FREETMPS;
	LEAVE;

	return GNOME_VIEW(SvGtkObjectRef(result, 0));
}

void
generic_foreach_view (GnomeView *view, void *data) {
	AV * args = (AV*)data;
	SV * handler = *av_fetch(args, 0, 0);
	int i;
	dSP;

	PUSHMARK(sp);
	XPUSHs(sv_2mortal(newSVGtkObjectRef(view, 0)));
	for (i=1;i<=av_len(args);i++)
		XPUSHs(sv_2mortal(newSVsv(*av_fetch(args, i, 0))));
	PUTBACK;
	perl_call_sv(handler, G_DISCARD);
	return;
}

void
generic_foreach_item (GnomeCanvasComponent *comp, void *data) {
	AV * args = (AV*)data;
	SV * handler = *av_fetch(args, 0, 0);
	int i;
	dSP;

	PUSHMARK(sp);
	XPUSHs(sv_2mortal(newSVGtkObjectRef(comp, 0)));
	for (i=1;i<=av_len(args);i++)
		XPUSHs(sv_2mortal(newSVsv(*av_fetch(args, i, 0))));
	PUTBACK;
	perl_call_sv(handler, G_DISCARD);
	return;
}

MODULE = Gnome::Embeddable		PACKAGE = Gnome::Embeddable		PREFIX = gnome_embeddable_

#ifdef GNOME_EMBEDDABLE

Gnome::Embeddable
gnome_embeddable_new (Class, handler, ...)
	SV *	Class
	SV *	handler
	CODE:
	{
		AV * args = newAV();

		PackCallbackST(args, 1);
		
		RETVAL = GNOME_EMBEDDABLE(gnome_embeddable_new(generic_view_factory, (gpointer)args));
	}
	OUTPUT:
	RETVAL

Gnome::Embeddable
gnome_embeddable_new_canvas_item (Class, handler, ...)
	SV *	Class
	SV *	handler
	CODE:
	{
		AV * args = newAV();

		PackCallbackST(args, 1);
		
		RETVAL = GNOME_EMBEDDABLE(gnome_embeddable_new_canvas_item(generic_canvas_factory, (gpointer)args));
	}
	OUTPUT:
	RETVAL

CORBA::Object
gnome_embeddable_corba_object_create (object)
	Gnome::Object	object

void
gnome_embeddable_add_verb (embeddable, name, label, hint)
	Gnome::Embeddable	embeddable
	char *	name
	char *	label
	char *	hint

#missing: add_verbs

void
gnome_embeddable_remove_verb (embeddable, name)
	Gnome::Embeddable	embeddable
	char *	name

void
gnome_embeddable_set_view_factory (embeddable, handler, ...)
	Gnome::Embeddable	embeddable
	SV *	handler
	CODE:
	{
		AV * args = newAV();

		PackCallbackST(args, 1);
		gnome_embeddable_set_view_factory(embeddable, generic_view_factory, (gpointer)args);
	}

#missing: get_verbs

char*
gnome_embeddable_get_uri (embeddable)
	Gnome::Embeddable	embeddable

void
gnome_embeddable_set_uri (embeddable, uri)
	Gnome::Embeddable	embeddable
	char *	uri

void
gnome_embeddable_foreach_view (embeddable, handler, ...)
	Gnome::Embeddable	embeddable
	SV *	handler
	CODE:
	{
		AV * args = newAV();

		PackCallbackST(args, 1);
		gnome_embeddable_foreach_view(embeddable, generic_foreach_view, (gpointer)args);
		/* FIXME: free args */
		SvREFCNT_dec(args);

	}

void
gnome_embeddable_foreach_item (embeddable, handler, ...)
	Gnome::Embeddable	embeddable
	SV *	handler
	CODE:
	{
		AV * args = newAV();

		PackCallbackST(args, 1);
		gnome_embeddable_foreach_view(embeddable, generic_foreach_item, (gpointer)args);
		/* FIXME: free args */
		SvREFCNT_dec(args);

	}

#endif

